#!/bin/sh
#OnTheFly SFS-load-uload to main unionfs branch. sfs-file must be out of the current unionfs branch to be loaded. Tested in pUPnGO and Puppy P412.

if [ $(which dialogfunctions.sh) ]; then #check functions present
INCLUDE=$(which dialogfunctions.sh)
else
echo "You are missing file 'dialogfunctions.sh'.
Program will not run without it."
exit
fi

CURDIR=$(pwd)
. "${INCLUDE}"	#contains functions used for the dialogs

TITLE="OTF SFS-loader"

create_loop () {
# find free loop device (below code from SFS-TCZ_Linker-2.1-T)
free_loop=""
num_loops=$(ls -al /dev/loop* | wc -l)
current_loops=$(expr $num_loops - 1)
for n in $current_loops; do
  aloop="/dev/loop$n"
  sfs_loaded=$(losetup-FULL $aloop 2>/dev/null)
  status=$?
  sfs_loaded=$(echo $sfs_loaded | sed 's/\(.*\/\)\(.*\)\(\.sfs)\)/\2/g') #'geany
  if [ $status -gt 0 -a -z "$free_loop" ]; then
    free_loop=$aloop
    break
  fi
  last_loop=$n
done

# create loop device if none found.
if [ -z "$free_loop" ]; then
  next_loop=$(expr $last_loop + 1)
  free_loop="/dev/loop$next_loop"
  echo "Creating new loop $next_loop..."
  [ ! -b $free_loop ] && mknod -m660 $free_loop b 7 $next_loop
  # verify loop was successfully created    
  if [ ! -b $free_loop ]; then
    TEXT="Failed! No loop devices available."
    message_fc
    return 1
  fi
  chown root.root $free_loop
fi
}

main_menu () {
TEXT="Please choose to load or unload SFS-file."
echo "Exit/Cancel
Load SFS-file
Unload SFS-file
Advanced functions" > /tmp/choices
menu_fc
case $REPLY_NUM in
  0) exit;;
  1) load_sfs;;
  2) unload_sfs;;
  3) advance_menu;;
esac
}

load_sfs (){
TEXT="Please choose SFS-file."
fileselect_fc
[ "${REPLY_NUM}" = "cancel" ] && return
FILENAME=$(basename $REPLY_CHAR)
if [ ! "${FILENAME/*./}" = "sfs" ]; then	#chech file type
  TEXT="Seems not to be a sfs-file."
  message_fc
  break
fi
NEWDIRNAME="${FILENAME%.*}"
MOUNTED=$(df | grep $NEWDIRNAME)
if [ ! "${MOUNTED}" = "" ]; then
  TEXT="File seems already mounted. Refuse to mount again."
  message_fc
  return
fi
mkdir -p /tmp/sfs/$NEWDIRNAME
mount -t squashfs -o loop $REPLY_CHAR /tmp/sfs/$NEWDIRNAME
if [ ! $? -eq 0 ]; then
  create_loop
  if [ ! $? -eq 0 ]; then
    TEXT="sfs-file could not be mounted - you might not have
any free 'loop'-devices back. Try booting with parameter
'max_loop=28'. Or add the following to /etc/modprobe.conf:
options loop max_loop=28.
Any number will work (I think) up to 255...
Alternatively unload some of the sfs-files already loaded."
    message_fc
    rmdir /tmp/sfs/$NEWDIRNAME
    return
  else
    mount -t squashfs -o loop $REPLY_CHAR /tmp/sfs/$NEWDIRNAME
    if [ ! $? -eq 0 ]; then
      TEXT="sfs-file could not be mounted - you might not have
any free 'loop'-devices back. Try booting with parameter
'max_loop=28'. Or add the following to /etc/modprobe.conf:
options loop max_loop=28.
Any number will work (I think) up to 255...
Alternatively unload some of the sfs-files already loaded."
      message_fc
      rmdir /tmp/sfs/$NEWDIRNAME
      return
    fi
  fi
fi
#need a check here to avoid no loops left for mounting
mount -o remount,append:/tmp/sfs/$NEWDIRNAME=ro /
if [ ! $? -eq 0 ]; then
  TEXT="sfs-file could not be appended - please make sure
that the sfs-file is not placed in the current branch."
  message_fc
  umount /tmp/sfs/$NEWDIRNAME
  rmdir /tmp/sfs/$NEWDIRNAME
else
  TEXT="$REPLY_CHAR succesfully appended to the unionfs."
  message_fc
fi
sync
}

unload_sfs () {
MOUNTED=$(ls /tmp/sfs)	
if [ "${MOUNTED}" = "" ]; then
  TEXT="No sfs-files seems to be loaded."
  message_fc
  return
else
  TEXT="Choose which sfs-file to unload:"
  echo "Cancel
  $MOUNTED" > /tmp/choices
  menu_fc
  [ "$REPLY_CHAR" = "Cancel" ] && return
  mount -o remount,del:/tmp/sfs/$REPLY_CHAR /
  if [ ! $? -eq 0 ]; then
    TEXT="sfs-file - $REPLY_CHAR - could not be unloaded.
Please make sure that no programs are using the content."
    message_fc
  else
    umount /tmp/sfs/$REPLY_CHAR
    rmdir /tmp/sfs/$REPLY_CHAR
    sync
    TEXT="sfs-file succesfully unloaded from the unionfs."
    message_fc
  fi
fi
}

advance_menu () {
TEXT="Choose action below."
echo "Cancel
Unload sfs files loaded at boot" > /tmp/choices
menu_fc
[ $REPLY_NUM -eq 0 ] && return
[ $REPLY_NUM -eq 1 ] && unload_sfs_adv
}

unload_sfs_adv () {
TEXT="Below is listed the sfs-files loaded at boot-time
(or by you later...)
Warning: You can do serios harm to your system!
Please choose which to unload."
BOOTMNT=$(mount | grep unionfs | cut -d ":" -f 2-10 | sed 's/=ro/\n/g' | sed 's/)//g' | tr -d ":" | tr " " "\n") #'geany
echo "Cancel" > /tmp/choices
for w in $BOOTMNT; do
TEMP1=$(df | grep $w | cut -d " " -f1)
TEMP2=$(losetup-FULL $TEMP1 | cut -d " " -f 3 | sed 's/)//g' | sed 's/(//g'  ) #'geany
echo "$TEMP2 on $w" >> /tmp/choices
done
menu_fc
[ "${REPLY_CHAR}" = "Cancel" ] && return
FILE2UNLOAD=$(echo $REPLY_CHAR | cut -d " " -f 3)
#unload file
mount -o remount,del:$FILE2UNLOAD /
if [ ! $? -eq 0 ]; then
  TEXT="sfs-file - $FILE2UNLOAD - could not be unloaded.
Please make sure that no programs are using the content."
  message_fc
else
TEMP1=$(df | grep $FILE2UNLOAD | cut -d " " -f1)
LOOP2UMOUNT=$(losetup-FULL $TEMP1 | cut -d " " -f 1 | sed 's/://g')
#umount loop
umount $LOOP2UMOUNT
TEXT="sfs-file succesfully unloaded from the unionfs."
message_fc
fi
}

#check if unionfs is aufs
CHECKUFS=$(mount | grep unionfs | grep aufs)

if [ "${CHECKUFS}" = "" ]; then
TEXT="Sorry. This wizard can only work with aufs."
message_fc
exit
fi

while :; do
main_menu
done